<?php

namespace Drupal\pf_alerta\Plugin\PushFrameworkChannel;

use Drupal;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\push_framework\ChannelBase;
use Drupal\user\UserInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;

/**
 * Plugin implementation of the push framework channel.
 *
 * @ChannelPlugin(
 *   id = "alerta",
 *   label = @Translation("Alerta"),
 *   description = @Translation("Provides the Alerta channel plugin.")
 * )
 */
class Alerta extends ChannelBase {

  /**
   * @var bool
   */
  protected $active;

  /**
   * @var string
   */
  protected $url;

  /**
   * @var string
   */
  protected $apikey;

  /**
   * @var string
   */
  protected $environment;

  /**
   * @var string
   */
  protected $host;

  /**
   * @var string
   */
  protected $pid;

  /**
   * @inheritDoc
   */
  public function init(): void {
    $config = $this->configFactory->get('pf_alerta.settings');
    $this->active = (bool) $config->get('active');
    $this->url = $config->get('url');
    $this->apikey = $config->get('apikey');
    $this->environment = $config->get('environment');
    $this->host = $config->get('host');
    $this->pid = $config->get('pid');
  }

  /**
   * {@inheritdoc}
   */
  public function applicable(UserInterface $user): bool {
    // This channel sends directly, so it never applies to other notifications.
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function send(UserInterface $user, ContentEntityInterface $entity, $attempt): string {
    if (!$this->active) {
      return self::RESULT_STATUS_FAILED;
    }
    $body = [
      'origin' => $this->host,
      'timeout' => 86400,
      'environment' => $this->environment,
      'resource' => 'undefined-' . Drupal::time()->getRequestTime(),
      'event' => 'Exception',
      'tags' => ['environment:' . $this->environment],
      'group' => 'Application',
      'severity' => 'critical',
      'service' => ['www.example.com'],
      'text' => '',
      'value' => '',
      'rawData' => '',
    ];
    if (!empty($this->pid)) {
      $body['attributes'] = [
        'gitlab_project_id' => $this->pid,
      ];
    }

    if (method_exists($entity, 'getPayload')) {
      $payload = $entity->getPayload();
      if (method_exists($payload, 'formatAlerta')) {
        $payload->formatAlerta($body);
      }
    }

    $options = [
      'headers' => [
        'Authorization' => 'Key ' . $this->apikey,
        'Content-Type' => 'application/json',
      ],
      'body' => json_encode($body),
    ];

    $client = new Client(['base_uri' => $this->url . '/alert']);
    try {
      $response = $client->request('POST', NULL, $options);
      $statusCode = $response->getStatusCode();
      if ($statusCode === 201) {
        return self::RESULT_STATUS_SUCCESS;
      }
    }
    catch (GuzzleException $e) {
      // TODO: Log this exception.
    }
    return self::RESULT_STATUS_FAILED;
  }

}
